@@ -16,11 +16,11 @@
  * plan trees.
  *
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/nodes/equalfuncs.c,v 1.108.2.1 2002/02/26 23:48:42 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/nodes/equalfuncs.c,v 1.161 2002/10/14 22:14:34 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -130,6 +130,8 @@ _equalOper(Oper *a, Oper *b)
 		return false;
 	if (a->opresulttype != b->opresulttype)
 		return false;
+	if (a->opretset != b->opretset)
+		return false;
 
 	/*
 	 * We do not examine opid or op_fcache, since these are logically
@@ -139,7 +141,8 @@ _equalOper(Oper *a, Oper *b)
 	 * (Besides, op_fcache is executor state, which we don't check --- see
 	 * notes at head of file.)
 	 *
-	 * It's probably not really necessary to check opresulttype either...
+	 * It's probably not really necessary to check opresulttype or opretset,
+	 * either...
 	 */
 
 	return true;
@@ -210,8 +213,19 @@ _equalFunc(Func *a, Func *b)
 {
 	if (a->funcid != b->funcid)
 		return false;
-	if (a->functype != b->functype)
+	if (a->funcresulttype != b->funcresulttype)
 		return false;
+	if (a->funcretset != b->funcretset)
+		return false;
+	/*
+	 * Special-case COERCE_DONTCARE, so that pathkeys can build coercion
+	 * nodes that are equal() to both explicit and implicit coercions.
+	 */
+	if (a->funcformat != b->funcformat &&
+		a->funcformat != COERCE_DONTCARE &&
+		b->funcformat != COERCE_DONTCARE)
+		return false;
+
 	/* Note we do not look at func_fcache; see notes for _equalOper */
 
 	return true;
@@ -220,9 +234,7 @@ _equalFunc(Func *a, Func *b)
 static bool
 _equalAggref(Aggref *a, Aggref *b)
 {
-	if (strcmp(a->aggname, b->aggname) != 0)
-		return false;
-	if (a->basetype != b->basetype)
+	if (a->aggfnoid != b->aggfnoid)
 		return false;
 	if (a->aggtype != b->aggtype)
 		return false;
@@ -255,7 +267,7 @@ _equalSubLink(SubLink *a, SubLink *b)
 static bool
 _equalArrayRef(ArrayRef *a, ArrayRef *b)
 {
-	if (a->refelemtype != b->refelemtype)
+	if (a->refrestype != b->refrestype)
 		return false;
 	if (a->refattrlength != b->refattrlength)
 		return false;
@@ -263,13 +275,17 @@ _equalArrayRef(ArrayRef *a, ArrayRef *b)
 		return false;
 	if (a->refelembyval != b->refelembyval)
 		return false;
+	if (a->refelemalign != b->refelemalign)
+		return false;
 	if (!equal(a->refupperindexpr, b->refupperindexpr))
 		return false;
 	if (!equal(a->reflowerindexpr, b->reflowerindexpr))
 		return false;
 	if (!equal(a->refexpr, b->refexpr))
 		return false;
-	return equal(a->refassgnexpr, b->refassgnexpr);
+	if (!equal(a->refassgnexpr, b->refassgnexpr))
+		return false;
+	return true;
 }
 
 static bool
@@ -295,6 +311,14 @@ _equalRelabelType(RelabelType *a, RelabelType *b)
 		return false;
 	if (a->resulttypmod != b->resulttypmod)
 		return false;
+	/*
+	 * Special-case COERCE_DONTCARE, so that pathkeys can build coercion
+	 * nodes that are equal() to both explicit and implicit coercions.
+	 */
+	if (a->relabelformat != b->relabelformat &&
+		a->relabelformat != COERCE_DONTCARE &&
+		b->relabelformat != COERCE_DONTCARE)
+		return false;
 	return true;
 }
 
@@ -335,9 +359,7 @@ _equalJoinExpr(JoinExpr *a, JoinExpr *b)
 		return false;
 	if (!equal(a->alias, b->alias))
 		return false;
-	if (!equal(a->colnames, b->colnames))
-		return false;
-	if (!equal(a->colvars, b->colvars))
+	if (a->rtindex != b->rtindex)
 		return false;
 
 	return true;
@@ -542,32 +564,6 @@ _equalJoinInfo(JoinInfo *a, JoinInfo *b)
 	return true;
 }
 
-static bool
-_equalIter(Iter *a, Iter *b)
-{
-	return equal(a->iterexpr, b->iterexpr);
-}
-
-static bool
-_equalStream(Stream *a, Stream *b)
-{
-	if (a->clausetype != b->clausetype)
-		return false;
-	if (a->groupup != b->groupup)
-		return false;
-	if (a->groupcost != b->groupcost)
-		return false;
-	if (a->groupsel != b->groupsel)
-		return false;
-	if (!equal(a->pathptr, b->pathptr))
-		return false;
-	if (!equal(a->cinfo, b->cinfo))
-		return false;
-	if (!equal(a->upstream, b->upstream))
-		return false;
-	return equal(a->downstream, b->downstream);
-}
-
 /*
  * Stuff from parsenodes.h
  */
@@ -577,23 +573,22 @@ _equalQuery(Query *a, Query *b)
 {
 	if (a->commandType != b->commandType)
 		return false;
+	if (a->querySource != b->querySource)
+		return false;
 	if (!equal(a->utilityStmt, b->utilityStmt))
 		return false;
 	if (a->resultRelation != b->resultRelation)
 		return false;
-	if (!equalstr(a->into, b->into))
+	if (!equal(a->into, b->into))
 		return false;
 	if (a->isPortal != b->isPortal)
 		return false;
 	if (a->isBinary != b->isBinary)
 		return false;
-	if (a->isTemp != b->isTemp)
-		return false;
 	if (a->hasAggs != b->hasAggs)
 		return false;
 	if (a->hasSubLinks != b->hasSubLinks)
 		return false;
-	/* we deliberately ignore originalQuery */
 	if (!equal(a->rtable, b->rtable))
 		return false;
 	if (!equal(a->jointree, b->jointree))
@@ -631,7 +626,7 @@ _equalQuery(Query *a, Query *b)
 static bool
 _equalInsertStmt(InsertStmt *a, InsertStmt *b)
 {
-	if (!equalstr(a->relname, b->relname))
+	if (!equal(a->relation, b->relation))
 		return false;
 	if (!equal(a->cols, b->cols))
 		return false;
@@ -646,12 +641,10 @@ _equalInsertStmt(InsertStmt *a, InsertStmt *b)
 static bool
 _equalDeleteStmt(DeleteStmt *a, DeleteStmt *b)
 {
-	if (!equalstr(a->relname, b->relname))
+	if (!equal(a->relation, b->relation))
 		return false;
 	if (!equal(a->whereClause, b->whereClause))
 		return false;
-	if (a->inhOpt != b->inhOpt)
-		return false;
 
 	return true;
 }
@@ -659,7 +652,7 @@ _equalDeleteStmt(DeleteStmt *a, DeleteStmt *b)
 static bool
 _equalUpdateStmt(UpdateStmt *a, UpdateStmt *b)
 {
-	if (!equalstr(a->relname, b->relname))
+	if (!equal(a->relation, b->relation))
 		return false;
 	if (!equal(a->targetList, b->targetList))
 		return false;
@@ -667,8 +660,6 @@ _equalUpdateStmt(UpdateStmt *a, UpdateStmt *b)
 		return false;
 	if (!equal(a->fromClause, b->fromClause))
 		return false;
-	if (a->inhOpt != b->inhOpt)
-		return false;
 
 	return true;
 }
@@ -678,9 +669,7 @@ _equalSelectStmt(SelectStmt *a, SelectStmt *b)
 {
 	if (!equal(a->distinctClause, b->distinctClause))
 		return false;
-	if (!equalstr(a->into, b->into))
-		return false;
-	if (a->istemp != b->istemp)
+	if (!equal(a->into, b->into))
 		return false;
 	if (!equal(a->intoColNames, b->intoColNames))
 		return false;
@@ -740,9 +729,7 @@ _equalAlterTableStmt(AlterTableStmt *a, AlterTableStmt *b)
 {
 	if (a->subtype != b->subtype)
 		return false;
-	if (!equalstr(a->relname, b->relname))
-		return false;
-	if (a->inhOpt != b->inhOpt)
+	if (!equal(a->relation, b->relation))
 		return false;
 	if (!equalstr(a->name, b->name))
 		return false;
@@ -759,9 +746,11 @@ _equalGrantStmt(GrantStmt *a, GrantStmt *b)
 {
 	if (a->is_grant != b->is_grant)
 		return false;
-	if (!equal(a->relnames, b->relnames))
+	if (a->objtype != b->objtype)
 		return false;
-	if (!equalstr(a->privileges, b->privileges))
+	if (!equal(a->objects, b->objects))
+		return false;
+	if (!equali(a->privileges, b->privileges))
 		return false;
 	if (!equal(a->grantees, b->grantees))
 		return false;
@@ -776,6 +765,19 @@ _equalPrivGrantee(PrivGrantee *a, PrivGrantee *b)
 		&& equalstr(a->groupname, b->groupname);
 }
 
+static bool
+_equalFuncWithArgs(FuncWithArgs *a, FuncWithArgs *b)
+{
+	return equal(a->funcname, b->funcname)
+		&& equal(a->funcargs, b->funcargs);
+}
+
+static bool
+_equalInsertDefault(InsertDefault *a, InsertDefault *b)
+{
+	return true;
+}
+
 static bool
 _equalClosePortalStmt(ClosePortalStmt *a, ClosePortalStmt *b)
 {
@@ -788,7 +790,7 @@ _equalClosePortalStmt(ClosePortalStmt *a, ClosePortalStmt *b)
 static bool
 _equalClusterStmt(ClusterStmt *a, ClusterStmt *b)
 {
-	if (!equalstr(a->relname, b->relname))
+	if (!equal(a->relation, b->relation))
 		return false;
 	if (!equalstr(a->indexname, b->indexname))
 		return false;
@@ -799,19 +801,15 @@ _equalClusterStmt(ClusterStmt *a, ClusterStmt *b)
 static bool
 _equalCopyStmt(CopyStmt *a, CopyStmt *b)
 {
-	if (a->binary != b->binary)
+	if (!equal(a->relation, b->relation))
 		return false;
-	if (!equalstr(a->relname, b->relname))
+	if (!equal(a->attlist, b->attlist))
 		return false;
-	if (a->oids != b->oids)
-		return false;
-	if (a->direction != b->direction)
+	if (a->is_from != b->is_from)
 		return false;
 	if (!equalstr(a->filename, b->filename))
 		return false;
-	if (!equalstr(a->delimiter, b->delimiter))
-		return false;
-	if (!equalstr(a->null_print, b->null_print))
+	if (!equal(a->options, b->options))
 		return false;
 
 	return true;
@@ -820,43 +818,26 @@ _equalCopyStmt(CopyStmt *a, CopyStmt *b)
 static bool
 _equalCreateStmt(CreateStmt *a, CreateStmt *b)
 {
-	if (!equalstr(a->relname, b->relname))
+	if (!equal(a->relation, b->relation))
 		return false;
 	if (!equal(a->tableElts, b->tableElts))
 		return false;
-	if (!equal(a->inhRelnames, b->inhRelnames))
+	if (!equal(a->inhRelations, b->inhRelations))
 		return false;
 	if (!equal(a->constraints, b->constraints))
 		return false;
-	if (a->istemp != b->istemp)
-		return false;
 	if (a->hasoids != b->hasoids)
 		return false;
 
 	return true;
 }
 
-static bool
-_equalVersionStmt(VersionStmt *a, VersionStmt *b)
-{
-	if (!equalstr(a->relname, b->relname))
-		return false;
-	if (a->direction != b->direction)
-		return false;
-	if (!equalstr(a->fromRelname, b->fromRelname))
-		return false;
-	if (!equalstr(a->date, b->date))
-		return false;
-
-	return true;
-}
-
 static bool
 _equalDefineStmt(DefineStmt *a, DefineStmt *b)
 {
 	if (a->defType != b->defType)
 		return false;
-	if (!equalstr(a->defname, b->defname))
+	if (!equal(a->defnames, b->defnames))
 		return false;
 	if (!equal(a->definition, b->definition))
 		return false;
@@ -867,10 +848,12 @@ _equalDefineStmt(DefineStmt *a, DefineStmt *b)
 static bool
 _equalDropStmt(DropStmt *a, DropStmt *b)
 {
-	if (!equal(a->names, b->names))
+	if (!equal(a->objects, b->objects))
 		return false;
 	if (a->removeType != b->removeType)
 		return false;
+	if (a->behavior != b->behavior)
+		return false;
 
 	return true;
 }
@@ -878,7 +861,7 @@ _equalDropStmt(DropStmt *a, DropStmt *b)
 static bool
 _equalTruncateStmt(TruncateStmt *a, TruncateStmt *b)
 {
-	if (!equalstr(a->relName, b->relName))
+	if (!equal(a->relation, b->relation))
 		return false;
 
 	return true;
@@ -889,11 +872,9 @@ _equalCommentStmt(CommentStmt *a, CommentStmt *b)
 {
 	if (a->objtype != b->objtype)
 		return false;
-	if (!equalstr(a->objname, b->objname))
+	if (!equal(a->objname, b->objname))
 		return false;
-	if (!equalstr(a->objproperty, b->objproperty))
-		return false;
-	if (!equal(a->objlist, b->objlist))
+	if (!equal(a->objargs, b->objargs))
 		return false;
 	if (!equalstr(a->comment, b->comment))
 		return false;
@@ -921,7 +902,7 @@ _equalIndexStmt(IndexStmt *a, IndexStmt *b)
 {
 	if (!equalstr(a->idxname, b->idxname))
 		return false;
-	if (!equalstr(a->relname, b->relname))
+	if (!equal(a->relation, b->relation))
 		return false;
 	if (!equalstr(a->accessMethod, b->accessMethod))
 		return false;
@@ -935,27 +916,27 @@ _equalIndexStmt(IndexStmt *a, IndexStmt *b)
 		return false;
 	if (a->primary != b->primary)
 		return false;
+	if (a->isconstraint != b->isconstraint)
+		return false;
 
 	return true;
 }
 
 static bool
-_equalProcedureStmt(ProcedureStmt *a, ProcedureStmt *b)
+_equalCreateFunctionStmt(CreateFunctionStmt *a, CreateFunctionStmt *b)
 {
 	if (a->replace != b->replace)
 		return false;
-	if (!equalstr(a->funcname, b->funcname))
+	if (!equal(a->funcname, b->funcname))
 		return false;
 	if (!equal(a->argTypes, b->argTypes))
 		return false;
 	if (!equal(a->returnType, b->returnType))
 		return false;
+	if (!equal(a->options, b->options))
+		return false;
 	if (!equal(a->withClause, b->withClause))
 		return false;
-	if (!equal(a->as, b->as))
-		return false;
-	if (!equalstr(a->language, b->language))
-		return false;
 
 	return true;
 }
@@ -963,10 +944,12 @@ _equalProcedureStmt(ProcedureStmt *a, ProcedureStmt *b)
 static bool
 _equalRemoveAggrStmt(RemoveAggrStmt *a, RemoveAggrStmt *b)
 {
-	if (!equalstr(a->aggname, b->aggname))
+	if (!equal(a->aggname, b->aggname))
 		return false;
 	if (!equal(a->aggtype, b->aggtype))
 		return false;
+	if (a->behavior != b->behavior)
+		return false;
 
 	return true;
 }
@@ -974,10 +957,12 @@ _equalRemoveAggrStmt(RemoveAggrStmt *a, RemoveAggrStmt *b)
 static bool
 _equalRemoveFuncStmt(RemoveFuncStmt *a, RemoveFuncStmt *b)
 {
-	if (!equalstr(a->funcname, b->funcname))
+	if (!equal(a->funcname, b->funcname))
 		return false;
 	if (!equal(a->args, b->args))
 		return false;
+	if (a->behavior != b->behavior)
+		return false;
 
 	return true;
 }
@@ -985,26 +970,40 @@ _equalRemoveFuncStmt(RemoveFuncStmt *a, RemoveFuncStmt *b)
 static bool
 _equalRemoveOperStmt(RemoveOperStmt *a, RemoveOperStmt *b)
 {
-	if (!equalstr(a->opname, b->opname))
+	if (!equal(a->opname, b->opname))
 		return false;
 	if (!equal(a->args, b->args))
 		return false;
+	if (a->behavior != b->behavior)
+		return false;
 
 	return true;
 }
 
+static bool
+_equalRemoveOpClassStmt(RemoveOpClassStmt *a, RemoveOpClassStmt *b)
+{
+	if (!equal(a->opclassname, b->opclassname))
+		return false;
+	if (!equalstr(a->amname, b->amname))
+		return false;
+	if (a->behavior != b->behavior)
+		return false;
+
+	return true;
+}
 
 static bool
 _equalRenameStmt(RenameStmt *a, RenameStmt *b)
 {
-	if (!equalstr(a->relname, b->relname))
+	if (!equal(a->relation, b->relation))
 		return false;
-	if (a->inhOpt != b->inhOpt)
-		return false;
-	if (!equalstr(a->column, b->column))
+	if (!equalstr(a->oldname, b->oldname))
 		return false;
 	if (!equalstr(a->newname, b->newname))
 		return false;
+	if (a->renameType != b->renameType)
+		return false;
 
 	return true;
 }
@@ -1012,16 +1011,18 @@ _equalRenameStmt(RenameStmt *a, RenameStmt *b)
 static bool
 _equalRuleStmt(RuleStmt *a, RuleStmt *b)
 {
+	if (!equal(a->relation, b->relation))
+		return false;
 	if (!equalstr(a->rulename, b->rulename))
 		return false;
 	if (!equal(a->whereClause, b->whereClause))
 		return false;
 	if (a->event != b->event)
 		return false;
-	if (!equal(a->object, b->object))
-		return false;
 	if (a->instead != b->instead)
 		return false;
+	if (a->replace != b->replace)
+		return false;
 	if (!equal(a->actions, b->actions))
 		return false;
 
@@ -1031,7 +1032,7 @@ _equalRuleStmt(RuleStmt *a, RuleStmt *b)
 static bool
 _equalNotifyStmt(NotifyStmt *a, NotifyStmt *b)
 {
-	if (!equalstr(a->relname, b->relname))
+	if (!equal(a->relation, b->relation))
 		return false;
 
 	return true;
@@ -1040,7 +1041,7 @@ _equalNotifyStmt(NotifyStmt *a, NotifyStmt *b)
 static bool
 _equalListenStmt(ListenStmt *a, ListenStmt *b)
 {
-	if (!equalstr(a->relname, b->relname))
+	if (!equal(a->relation, b->relation))
 		return false;
 
 	return true;
@@ -1049,7 +1050,7 @@ _equalListenStmt(ListenStmt *a, ListenStmt *b)
 static bool
 _equalUnlistenStmt(UnlistenStmt *a, UnlistenStmt *b)
 {
-	if (!equalstr(a->relname, b->relname))
+	if (!equal(a->relation, b->relation))
 		return false;
 
 	return true;
@@ -1060,6 +1061,19 @@ _equalTransactionStmt(TransactionStmt *a, TransactionStmt *b)
 {
 	if (a->command != b->command)
 		return false;
+	if (!equal(a->options, b->options))
+		return false;
+
+	return true;
+}
+
+static bool
+_equalCompositeTypeStmt(CompositeTypeStmt *a, CompositeTypeStmt *b)
+{
+	if (!equal(a->typevar, b->typevar))
+		return false;
+	if (!equal(a->coldeflist, b->coldeflist))
+		return false;
 
 	return true;
 }
@@ -1067,12 +1081,14 @@ _equalTransactionStmt(TransactionStmt *a, TransactionStmt *b)
 static bool
 _equalViewStmt(ViewStmt *a, ViewStmt *b)
 {
-	if (!equalstr(a->viewname, b->viewname))
+	if (!equal(a->view, b->view))
 		return false;
 	if (!equal(a->aliases, b->aliases))
 		return false;
 	if (!equal(a->query, b->query))
 		return false;
+	if (a->replace != b->replace)
+		return false;
 
 	return true;
 }
@@ -1086,16 +1102,74 @@ _equalLoadStmt(LoadStmt *a, LoadStmt *b)
 	return true;
 }
 
+static bool
+_equalCreateDomainStmt(CreateDomainStmt *a, CreateDomainStmt *b)
+{
+	if (!equal(a->domainname, b->domainname))
+		return false;
+	if (!equal(a->typename, b->typename))
+		return false;
+	if (!equal(a->constraints, b->constraints))
+		return false;
+
+	return true;
+}
+
+static bool
+_equalCreateOpClassStmt(CreateOpClassStmt *a, CreateOpClassStmt *b)
+{
+	if (!equal(a->opclassname, b->opclassname))
+		return false;
+	if (!equalstr(a->amname, b->amname))
+		return false;
+	if (!equal(a->datatype, b->datatype))
+		return false;
+	if (!equal(a->items, b->items))
+		return false;
+	if (a->isDefault != b->isDefault)
+		return false;
+
+	return true;
+}
+
+static bool
+_equalCreateOpClassItem(CreateOpClassItem *a, CreateOpClassItem *b)
+{
+	if (a->itemtype != b->itemtype)
+		return false;
+	if (!equal(a->name, b->name))
+		return false;
+	if (!equal(a->args, b->args))
+		return false;
+	if (a->number != b->number)
+		return false;
+	if (a->recheck != b->recheck)
+		return false;
+	if (!equal(a->storedtype, b->storedtype))
+		return false;
+
+	return true;
+}
+
 static bool
 _equalCreatedbStmt(CreatedbStmt *a, CreatedbStmt *b)
 {
 	if (!equalstr(a->dbname, b->dbname))
 		return false;
-	if (!equalstr(a->dbpath, b->dbpath))
+	if (!equal(a->options, b->options))
 		return false;
-	if (!equalstr(a->dbtemplate, b->dbtemplate))
+
+	return true;
+}
+
+static bool
+_equalAlterDatabaseSetStmt(AlterDatabaseSetStmt *a, AlterDatabaseSetStmt *b)
+{
+	if (!equalstr(a->dbname, b->dbname))
 		return false;
-	if (a->encoding != b->encoding)
+	if (!equalstr(a->variable, b->variable))
+		return false;
+	if (!equal(a->value, b->value))
 		return false;
 
 	return true;
@@ -1123,7 +1197,7 @@ _equalVacuumStmt(VacuumStmt *a, VacuumStmt *b)
 		return false;
 	if (a->verbose != b->verbose)
 		return false;
-	if (!equalstr(a->vacrel, b->vacrel))
+	if (!equal(a->relation, b->relation))
 		return false;
 	if (!equal(a->va_cols, b->va_cols))
 		return false;
@@ -1147,7 +1221,7 @@ _equalExplainStmt(ExplainStmt *a, ExplainStmt *b)
 static bool
 _equalCreateSeqStmt(CreateSeqStmt *a, CreateSeqStmt *b)
 {
-	if (!equalstr(a->seqname, b->seqname))
+	if (!equal(a->sequence, b->sequence))
 		return false;
 	if (!equal(a->options, b->options))
 		return false;
@@ -1162,6 +1236,8 @@ _equalVariableSetStmt(VariableSetStmt *a, VariableSetStmt *b)
 		return false;
 	if (!equal(a->args, b->args))
 		return false;
+	if (a->is_local != b->is_local)
+		return false;
 
 	return true;
 }
@@ -1189,9 +1265,9 @@ _equalCreateTrigStmt(CreateTrigStmt *a, CreateTrigStmt *b)
 {
 	if (!equalstr(a->trigname, b->trigname))
 		return false;
-	if (!equalstr(a->relname, b->relname))
+	if (!equal(a->relation, b->relation))
 		return false;
-	if (!equalstr(a->funcname, b->funcname))
+	if (!equal(a->funcname, b->funcname))
 		return false;
 	if (!equal(a->args, b->args))
 		return false;
@@ -1215,18 +1291,22 @@ _equalCreateTrigStmt(CreateTrigStmt *a, CreateTrigStmt *b)
 		return false;
 	if (a->initdeferred != b->initdeferred)
 		return false;
-	if (!equalstr(a->constrrelname, b->constrrelname))
+	if (!equal(a->constrrel, b->constrrel))
 		return false;
 
 	return true;
 }
 
 static bool
-_equalDropTrigStmt(DropTrigStmt *a, DropTrigStmt *b)
+_equalDropPropertyStmt(DropPropertyStmt *a, DropPropertyStmt *b)
 {
-	if (!equalstr(a->trigname, b->trigname))
+	if (!equal(a->relation, b->relation))
 		return false;
-	if (!equalstr(a->relname, b->relname))
+	if (!equalstr(a->property, b->property))
+		return false;
+	if (a->removeType != b->removeType)
+		return false;
+	if (a->behavior != b->behavior)
 		return false;
 
 	return true;
@@ -1237,9 +1317,9 @@ _equalCreatePLangStmt(CreatePLangStmt *a, CreatePLangStmt *b)
 {
 	if (!equalstr(a->plname, b->plname))
 		return false;
-	if (!equalstr(a->plhandler, b->plhandler))
+	if (!equal(a->plhandler, b->plhandler))
 		return false;
-	if (!equalstr(a->plcompiler, b->plcompiler))
+	if (!equal(a->plvalidator, b->plvalidator))
 		return false;
 	if (a->pltrusted != b->pltrusted)
 		return false;
@@ -1252,6 +1332,8 @@ _equalDropPLangStmt(DropPLangStmt *a, DropPLangStmt *b)
 {
 	if (!equalstr(a->plname, b->plname))
 		return false;
+	if (a->behavior != b->behavior)
+		return false;
 
 	return true;
 }
@@ -1278,6 +1360,19 @@ _equalAlterUserStmt(AlterUserStmt *a, AlterUserStmt *b)
 	return true;
 }
 
+static bool
+_equalAlterUserSetStmt(AlterUserSetStmt *a, AlterUserSetStmt *b)
+{
+	if (!equalstr(a->user, b->user))
+		return false;
+	if (!equalstr(a->variable, b->variable))
+		return false;
+	if (!equal(a->value, b->value))
+		return false;
+
+	return true;
+}
+
 static bool
 _equalDropUserStmt(DropUserStmt *a, DropUserStmt *b)
 {
@@ -1290,7 +1385,7 @@ _equalDropUserStmt(DropUserStmt *a, DropUserStmt *b)
 static bool
 _equalLockStmt(LockStmt *a, LockStmt *b)
 {
-	if (!equal(a->rellist, b->rellist))
+	if (!equal(a->relations, b->relations))
 		return false;
 	if (a->mode != b->mode)
 		return false;
@@ -1347,6 +1442,8 @@ _equalReindexStmt(ReindexStmt *a, ReindexStmt *b)
 {
 	if (a->reindexType != b->reindexType)
 		return false;
+	if (!equal(a->relation, b->relation))
+		return false;
 	if (!equalstr(a->name, b->name))
 		return false;
 	if (a->force != b->force)
@@ -1357,12 +1454,107 @@ _equalReindexStmt(ReindexStmt *a, ReindexStmt *b)
 	return true;
 }
 
+static bool
+_equalCreateSchemaStmt(CreateSchemaStmt *a, CreateSchemaStmt *b)
+{
+	if (!equalstr(a->schemaname, b->schemaname))
+		return false;
+	if (!equalstr(a->authid, b->authid))
+		return false;
+	if (!equal(a->schemaElts, b->schemaElts))
+		return false;
+
+	return true;
+}
+
+static bool
+_equalCreateConversionStmt(CreateConversionStmt *a, CreateConversionStmt *b)
+{
+	if (!equal(a->conversion_name, b->conversion_name))
+		return false;
+	if (!equalstr(a->for_encoding_name, b->for_encoding_name))
+		return false;
+	if (!equalstr(a->to_encoding_name, b->to_encoding_name))
+		return false;
+	if (!equal(a->func_name, b->func_name))
+		return false;
+	if (a->def != b->def)
+		return false;
+
+	return true;
+}
+
+static bool
+_equalCreateCastStmt(CreateCastStmt *a, CreateCastStmt *b)
+{
+	if (!equal(a->sourcetype, b->sourcetype))
+		return false;
+	if (!equal(a->targettype, b->targettype))
+		return false;
+	if (!equal(a->func, b->func))
+		return false;
+	if (a->context != b->context)
+		return false;
+
+	return true;
+}
+
+static bool
+_equalDropCastStmt(DropCastStmt *a, DropCastStmt *b)
+{
+	if (!equal(a->sourcetype, b->sourcetype))
+		return false;
+	if (!equal(a->targettype, b->targettype))
+		return false;
+	if (a->behavior != b->behavior)
+		return false;
+
+	return true;
+}
+
+static bool
+_equalPrepareStmt(PrepareStmt *a, PrepareStmt *b)
+{
+	if (!equalstr(a->name, b->name))
+		return false;
+	if (!equal(a->argtypes, b->argtypes))
+		return false;
+	if (!equali(a->argtype_oids, b->argtype_oids))
+		return false;
+	if (!equal(a->query, b->query))
+		return false;
+
+	return true;
+}
+
+static bool
+_equalExecuteStmt(ExecuteStmt *a, ExecuteStmt *b)
+{
+	if (!equalstr(a->name, b->name))
+		return false;
+	if (!equal(a->into, b->into))
+		return false;
+	if (!equal(a->params, b->params))
+		return false;
+
+	return true;
+}
+
+static bool
+_equalDeallocateStmt(DeallocateStmt *a, DeallocateStmt *b)
+{
+	if (!equalstr(a->name, b->name))
+		return false;
+
+	return true;
+}
+
 static bool
 _equalAExpr(A_Expr *a, A_Expr *b)
 {
 	if (a->oper != b->oper)
 		return false;
-	if (!equalstr(a->opname, b->opname))
+	if (!equal(a->name, b->name))
 		return false;
 	if (!equal(a->lexpr, b->lexpr))
 		return false;
@@ -1373,13 +1565,22 @@ _equalAExpr(A_Expr *a, A_Expr *b)
 }
 
 static bool
-_equalAttr(Attr *a, Attr *b)
+_equalColumnRef(ColumnRef *a, ColumnRef *b)
 {
-	if (strcmp(a->relname, b->relname) != 0)
+	if (!equal(a->fields, b->fields))
 		return false;
-	if (!equal(a->paramNo, b->paramNo))
+	if (!equal(a->indirection, b->indirection))
 		return false;
-	if (!equal(a->attrs, b->attrs))
+
+	return true;
+}
+
+static bool
+_equalParamRef(ParamRef *a, ParamRef *b)
+{
+	if (a->number != b->number)
+		return false;
+	if (!equal(a->fields, b->fields))
 		return false;
 	if (!equal(a->indirection, b->indirection))
 		return false;
@@ -1398,36 +1599,10 @@ _equalAConst(A_Const *a, A_Const *b)
 	return true;
 }
 
-static bool
-_equalParamNo(ParamNo *a, ParamNo *b)
-{
-	if (a->number != b->number)
-		return false;
-	if (!equal(a->typename, b->typename))
-		return false;
-	if (!equal(a->indirection, b->indirection))
-		return false;
-
-	return true;
-}
-
-static bool
-_equalIdent(Ident *a, Ident *b)
-{
-	if (!equalstr(a->name, b->name))
-		return false;
-	if (!equal(a->indirection, b->indirection))
-		return false;
-	if (a->isRel != b->isRel)
-		return false;
-
-	return true;
-}
-
 static bool
 _equalFuncCall(FuncCall *a, FuncCall *b)
 {
-	if (!equalstr(a->funcname, b->funcname))
+	if (!equal(a->funcname, b->funcname))
 		return false;
 	if (!equal(a->args, b->args))
 		return false;
@@ -1450,6 +1625,19 @@ _equalAIndices(A_Indices *a, A_Indices *b)
 	return true;
 }
 
+static bool
+_equalExprFieldSelect(ExprFieldSelect *a, ExprFieldSelect *b)
+{
+	if (!equal(a->arg, b->arg))
+		return false;
+	if (!equal(a->fields, b->fields))
+		return false;
+	if (!equal(a->indirection, b->indirection))
+		return false;
+
+	return true;
+}
+
 static bool
 _equalResTarget(ResTarget *a, ResTarget *b)
 {
@@ -1477,7 +1665,7 @@ _equalTypeCast(TypeCast *a, TypeCast *b)
 static bool
 _equalSortGroupBy(SortGroupBy *a, SortGroupBy *b)
 {
-	if (!equalstr(a->useOp, b->useOp))
+	if (!equal(a->useOp, b->useOp))
 		return false;
 	if (!equal(a->node, b->node))
 		return false;
@@ -1485,14 +1673,31 @@ _equalSortGroupBy(SortGroupBy *a, SortGroupBy *b)
 	return true;
 }
 
+static bool
+_equalAlias(Alias *a, Alias *b)
+{
+	if (!equalstr(a->aliasname, b->aliasname))
+		return false;
+	if (!equal(a->colnames, b->colnames))
+		return false;
+
+	return true;
+}
+
 static bool
 _equalRangeVar(RangeVar *a, RangeVar *b)
 {
+	if (!equalstr(a->catalogname, b->catalogname))
+		return false;
+	if (!equalstr(a->schemaname, b->schemaname))
+		return false;
 	if (!equalstr(a->relname, b->relname))
 		return false;
 	if (a->inhOpt != b->inhOpt)
 		return false;
-	if (!equal(a->name, b->name))
+	if (a->istemp != b->istemp)
+		return false;
+	if (!equal(a->alias, b->alias))
 		return false;
 
 	return true;
@@ -1503,7 +1708,20 @@ _equalRangeSubselect(RangeSubselect *a, RangeSubselect *b)
 {
 	if (!equal(a->subquery, b->subquery))
 		return false;
-	if (!equal(a->name, b->name))
+	if (!equal(a->alias, b->alias))
+		return false;
+
+	return true;
+}
+
+static bool
+_equalRangeFunction(RangeFunction *a, RangeFunction *b)
+{
+	if (!equal(a->funccallnode, b->funccallnode))
+		return false;
+	if (!equal(a->alias, b->alias))
+		return false;
+	if (!equal(a->coldeflist, b->coldeflist))
 		return false;
 
 	return true;
@@ -1512,12 +1730,16 @@ _equalRangeSubselect(RangeSubselect *a, RangeSubselect *b)
 static bool
 _equalTypeName(TypeName *a, TypeName *b)
 {
-	if (!equalstr(a->name, b->name))
+	if (!equal(a->names, b->names))
+		return false;
+	if (a->typeid != b->typeid)
 		return false;
 	if (a->timezone != b->timezone)
 		return false;
 	if (a->setof != b->setof)
 		return false;
+	if (a->pct_type != b->pct_type)
+		return false;
 	if (a->typmod != b->typmod)
 		return false;
 	if (!equal(a->arrayBounds, b->arrayBounds))
@@ -1531,9 +1753,11 @@ _equalIndexElem(IndexElem *a, IndexElem *b)
 {
 	if (!equalstr(a->name, b->name))
 		return false;
+	if (!equal(a->funcname, b->funcname))
+		return false;
 	if (!equal(a->args, b->args))
 		return false;
-	if (!equalstr(a->class, b->class))
+	if (!equal(a->opclass, b->opclass))
 		return false;
 
 	return true;
@@ -1546,6 +1770,10 @@ _equalColumnDef(ColumnDef *a, ColumnDef *b)
 		return false;
 	if (!equal(a->typename, b->typename))
 		return false;
+	if (a->inhcount != b->inhcount)
+		return false;
+	if (a->is_local != b->is_local)
+		return false;
 	if (a->is_not_null != b->is_not_null)
 		return false;
 	if (!equal(a->raw_default, b->raw_default))
@@ -1554,6 +1782,8 @@ _equalColumnDef(ColumnDef *a, ColumnDef *b)
 		return false;
 	if (!equal(a->constraints, b->constraints))
 		return false;
+	if (!equal(a->support, b->support))
+		return false;
 
 	return true;
 }
@@ -1602,12 +1832,20 @@ _equalTargetEntry(TargetEntry *a, TargetEntry *b)
 static bool
 _equalRangeTblEntry(RangeTblEntry *a, RangeTblEntry *b)
 {
-	if (!equalstr(a->relname, b->relname))
+	if (a->rtekind != b->rtekind)
 		return false;
 	if (a->relid != b->relid)
 		return false;
 	if (!equal(a->subquery, b->subquery))
 		return false;
+	if (!equal(a->funcexpr, b->funcexpr))
+		return false;
+	if (!equal(a->coldeflist, b->coldeflist))
+		return false;
+	if (a->jointype != b->jointype)
+		return false;
+	if (!equal(a->joinaliasvars, b->joinaliasvars))
+		return false;
 	if (!equal(a->alias, b->alias))
 		return false;
 	if (!equal(a->eref, b->eref))
@@ -1642,20 +1880,24 @@ _equalFkConstraint(FkConstraint *a, FkConstraint *b)
 {
 	if (!equalstr(a->constr_name, b->constr_name))
 		return false;
-	if (!equalstr(a->pktable_name, b->pktable_name))
+	if (!equal(a->pktable, b->pktable))
 		return false;
 	if (!equal(a->fk_attrs, b->fk_attrs))
 		return false;
 	if (!equal(a->pk_attrs, b->pk_attrs))
 		return false;
-	if (!equalstr(a->match_type, b->match_type))
+	if (a->fk_matchtype != b->fk_matchtype)
 		return false;
-	if (a->actions != b->actions)
+	if (a->fk_upd_action != b->fk_upd_action)
+		return false;
+	if (a->fk_del_action != b->fk_del_action)
 		return false;
 	if (a->deferrable != b->deferrable)
 		return false;
 	if (a->initdeferred != b->initdeferred)
 		return false;
+	if (a->skip_validation != b->skip_validation)
+		return false;
 
 	return true;
 }
@@ -1706,6 +1948,20 @@ _equalBooleanTest(BooleanTest *a, BooleanTest *b)
 	return true;
 }
 
+static bool
+_equalConstraintTest(ConstraintTest *a, ConstraintTest *b)
+{
+	if (!equal(a->arg, b->arg))
+		return false;
+	if (a->testtype != b->testtype)
+		return false;
+	if (!equalstr(a->name, b->name))
+		return false;
+	if (!equal(a->check_expr, b->check_expr))
+		return false;
+	return true;
+}
+
 /*
  * Stuff from pg_list.h
  */
@@ -1724,7 +1980,11 @@ _equalValue(Value *a, Value *b)
 		case T_String:
 		case T_BitString:
 			return strcmp(a->val.str, b->val.str) == 0;
+		case T_Null:
+			/* nothing to do */
+			break;
 		default:
+			elog(ERROR, "_equalValue: unknown node type %d", a->type);
 			break;
 	}
 
@@ -1797,9 +2057,6 @@ equal(void *a, void *b)
 		case T_ArrayRef:
 			retval = _equalArrayRef(a, b);
 			break;
-		case T_Iter:
-			retval = _equalIter(a, b);
-			break;
 		case T_RelabelType:
 			retval = _equalRelabelType(a, b);
 			break;
@@ -1840,9 +2097,6 @@ equal(void *a, void *b)
 		case T_JoinInfo:
 			retval = _equalJoinInfo(a, b);
 			break;
-		case T_Stream:
-			retval = _equalStream(a, b);
-			break;
 		case T_TidPath:
 			retval = _equalTidPath(a, b);
 			break;
@@ -1874,10 +2128,12 @@ equal(void *a, void *b)
 				retval = true;
 			}
 			break;
+
 		case T_Integer:
 		case T_Float:
 		case T_String:
 		case T_BitString:
+		case T_Null:
 			retval = _equalValue(a, b);
 			break;
 
@@ -1917,9 +2173,6 @@ equal(void *a, void *b)
 		case T_CreateStmt:
 			retval = _equalCreateStmt(a, b);
 			break;
-		case T_VersionStmt:
-			retval = _equalVersionStmt(a, b);
-			break;
 		case T_DefineStmt:
 			retval = _equalDefineStmt(a, b);
 			break;
@@ -1938,8 +2191,8 @@ equal(void *a, void *b)
 		case T_IndexStmt:
 			retval = _equalIndexStmt(a, b);
 			break;
-		case T_ProcedureStmt:
-			retval = _equalProcedureStmt(a, b);
+		case T_CreateFunctionStmt:
+			retval = _equalCreateFunctionStmt(a, b);
 			break;
 		case T_RemoveAggrStmt:
 			retval = _equalRemoveAggrStmt(a, b);
@@ -1950,6 +2203,9 @@ equal(void *a, void *b)
 		case T_RemoveOperStmt:
 			retval = _equalRemoveOperStmt(a, b);
 			break;
+		case T_RemoveOpClassStmt:
+			retval = _equalRemoveOpClassStmt(a, b);
+			break;
 		case T_RenameStmt:
 			retval = _equalRenameStmt(a, b);
 			break;
@@ -1968,15 +2224,30 @@ equal(void *a, void *b)
 		case T_TransactionStmt:
 			retval = _equalTransactionStmt(a, b);
 			break;
+		case T_CompositeTypeStmt:
+			retval = _equalCompositeTypeStmt(a, b);
+			break;
 		case T_ViewStmt:
 			retval = _equalViewStmt(a, b);
 			break;
 		case T_LoadStmt:
 			retval = _equalLoadStmt(a, b);
 			break;
+		case T_CreateDomainStmt:
+			retval = _equalCreateDomainStmt(a, b);
+			break;
+		case T_CreateOpClassStmt:
+			retval = _equalCreateOpClassStmt(a, b);
+			break;
+		case T_CreateOpClassItem:
+			retval = _equalCreateOpClassItem(a, b);
+			break;
 		case T_CreatedbStmt:
 			retval = _equalCreatedbStmt(a, b);
 			break;
+		case T_AlterDatabaseSetStmt:
+			retval = _equalAlterDatabaseSetStmt(a, b);
+			break;
 		case T_DropdbStmt:
 			retval = _equalDropdbStmt(a, b);
 			break;
@@ -2001,8 +2272,8 @@ equal(void *a, void *b)
 		case T_CreateTrigStmt:
 			retval = _equalCreateTrigStmt(a, b);
 			break;
-		case T_DropTrigStmt:
-			retval = _equalDropTrigStmt(a, b);
+		case T_DropPropertyStmt:
+			retval = _equalDropPropertyStmt(a, b);
 			break;
 		case T_CreatePLangStmt:
 			retval = _equalCreatePLangStmt(a, b);
@@ -2016,6 +2287,9 @@ equal(void *a, void *b)
 		case T_AlterUserStmt:
 			retval = _equalAlterUserStmt(a, b);
 			break;
+		case T_AlterUserSetStmt:
+			retval = _equalAlterUserSetStmt(a, b);
+			break;
 		case T_DropUserStmt:
 			retval = _equalDropUserStmt(a, b);
 			break;
@@ -2040,28 +2314,49 @@ equal(void *a, void *b)
 		case T_CheckPointStmt:
 			retval = true;
 			break;
+		case T_CreateSchemaStmt:
+			retval = _equalCreateSchemaStmt(a, b);
+			break;
+		case T_CreateConversionStmt:
+			retval = _equalCreateConversionStmt(a, b);
+			break;
+		case T_CreateCastStmt:
+			retval = _equalCreateCastStmt(a, b);
+			break;
+		case T_DropCastStmt:
+			retval = _equalDropCastStmt(a, b);
+			break;
+		case T_PrepareStmt:
+			retval = _equalPrepareStmt(a, b);
+			break;
+		case T_ExecuteStmt:
+			retval = _equalExecuteStmt(a, b);
+			break;
+		case T_DeallocateStmt:
+			retval = _equalDeallocateStmt(a, b);
+			break;
 
 		case T_A_Expr:
 			retval = _equalAExpr(a, b);
 			break;
-		case T_Attr:
-			retval = _equalAttr(a, b);
+		case T_ColumnRef:
+			retval = _equalColumnRef(a, b);
+			break;
+		case T_ParamRef:
+			retval = _equalParamRef(a, b);
 			break;
 		case T_A_Const:
 			retval = _equalAConst(a, b);
 			break;
-		case T_ParamNo:
-			retval = _equalParamNo(a, b);
-			break;
-		case T_Ident:
-			retval = _equalIdent(a, b);
-			break;
 		case T_FuncCall:
 			retval = _equalFuncCall(a, b);
 			break;
 		case T_A_Indices:
 			retval = _equalAIndices(a, b);
 			break;
+		case T_ExprFieldSelect:
+			retval = _equalExprFieldSelect(a, b);
+			break;
 		case T_ResTarget:
 			retval = _equalResTarget(a, b);
 			break;
@@ -2071,12 +2366,18 @@ equal(void *a, void *b)
 		case T_SortGroupBy:
 			retval = _equalSortGroupBy(a, b);
 			break;
+		case T_Alias:
+			retval = _equalAlias(a, b);
+			break;
 		case T_RangeVar:
 			retval = _equalRangeVar(a, b);
 			break;
 		case T_RangeSubselect:
 			retval = _equalRangeSubselect(a, b);
 			break;
+		case T_RangeFunction:
+			retval = _equalRangeFunction(a, b);
+			break;
 		case T_TypeName:
 			retval = _equalTypeName(a, b);
 			break;
@@ -2117,15 +2418,24 @@ equal(void *a, void *b)
 		case T_BooleanTest:
 			retval = _equalBooleanTest(a, b);
 			break;
+		case T_ConstraintTest:
+			retval = _equalConstraintTest(a, b);
+			break;
 		case T_FkConstraint:
 			retval = _equalFkConstraint(a, b);
 			break;
 		case T_PrivGrantee:
 			retval = _equalPrivGrantee(a, b);
 			break;
+		case T_FuncWithArgs:
+			retval = _equalFuncWithArgs(a, b);
+			break;
+		case T_InsertDefault:
+			retval = _equalInsertDefault(a, b);
+			break;
 
 		default:
-			elog(NOTICE, "equal: don't know whether nodes of type %d are equal",
+			elog(WARNING, "equal: don't know whether nodes of type %d are equal",
 				 nodeTag(a));
 			break;
 	}
